"use strict";

function SockWrapper(sock) {

  var that = this,
    messageCallbacks = new Map(),
    closeCallbacks = [],
    errorCallbacks = new Map(),
    rawDataCallback = null;

  this.onClose = function (callback) {
    closeCallbacks.push(callback);
  };

  this.onMessage = function (type, callback) {
    messageCallbacks.set(type, callback);
  };

  this.onRawData = function (callback) {
    rawDataCallback = callback;
  };

  this.onError = function (code, callback) {
    errorCallbacks.set(code, callback);
  };

  this.sendType = function (type, data) {
    sock.send(JSON.stringify({
      type : type, data : (data || {})
    }));
  };

  sock.onclose = function (e) {
    for (let it = 0; it < closeCallbacks.length; ++it) {
      closeCallbacks[it](e);
    }
  };

  sock.onmessage = function (message) {
    // Audio chunk for host room mode is the only binary message we get.
    // Everything else should be JSON.
    if (message.data instanceof Blob) {
      if (rawDataCallback) {
        rawDataCallback(message.data);
      }
      return;
    }

    let payload;
    try {
      payload = JSON.parse(message.data);
    } catch (err) {
      return showErrorAndDie("Server sent a non-JSON message.");
    }
    console.log("got message: ", payload);
    if (payload.hasOwnProperty("error")) {
      let code = payload.code;
      if (code === undefined || !errorCallbacks.has(code)) {
        return showErrorAndDie(payload.error);
      }
      return errorCallbacks.get(code)(payload.error);
    }
    if (payload.type === 'non-patient-firewall') {
      return that.sendType('non-patient-firewall', {when: myClock.clock()});
    }
    if (!messageCallbacks.has(payload.type)) {
      return console.log(
        "type '" + payload.type + "' not registered.", payload.data
      );
    }
    messageCallbacks.get(payload.type)(payload.data);
  };

};
